unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  StdCtrls, Menus, ExtCtrls;

type
  TMainDlg = class(TForm)
    Memo: TMemo;
    Panel1: TPanel;
    OpenBtn: TButton;
    PrettyPrintBtn: TButton;
    SearchReplaceBtn: TButton;
    procedure OpenBtnClick(Sender: TObject);
    procedure PrettyPrintBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SearchReplaceBtnClick(Sender: TObject);
  private
    FDocument : TStringList;

    procedure UpdateMemo;
  public
  end;

var
  MainDlg: TMainDlg;

implementation

{$R *.dfm}

uses
  XmlParser, XmlInterpreter, XmlInterpreterVisitors, Dialogs, SearchReplaceForm;


procedure TMainDlg.UpdateMemo;
begin
  Memo.Lines.Text := FDocument.Text;
end;


procedure TMainDlg.FormCreate(Sender: TObject);
begin
  FDocument := TStringList.Create;
end;


procedure TMainDlg.FormDestroy(Sender: TObject);
begin
  FDocument.Free;
end;


procedure TMainDlg.OpenBtnClick(Sender: TObject);
  var
    FileName : string;
begin
  if PromptForFileName(FileName,
                       'XML files (*.xml)|*.xml|CSV files (*.csv)|*.csv') then begin
    FDocument.LoadFromFile(FileName);
    UpdateMemo;
  end;
end;


procedure TMainDlg.SearchReplaceBtnClick(Sender: TObject);
  var
    Parser      : TXmlParser;
    Interpreter : TXmlInterpreter;
    Visitor     : TXmlPrettyPrinter;
begin
  with TSearchReplaceDlg.Create(Self) do begin
    Parser      := TXmlParser.Create;
    Interpreter := TXmlInterpreter.Create;
    Visitor     := TXmlPrettyPrinter.Create;

    try
      if Execute then begin
        Parser.Parse(FDocument.Text,Interpreter.XmlDoc);
        Interpreter.XmlDoc.SearchAndReplace(FindText,ReplaceText);
        Interpreter.XmlDoc.Accept(Visitor);
        FDocument.Text := Visitor.Text;
        UpdateMemo;
      end;
    finally
      Parser.Free;
      Interpreter.Free;
      Visitor.Free;
      Free;
    end;
  end;
end;


procedure TMainDlg.PrettyPrintBtnClick(Sender: TObject);
  var
    Parser      : TXmlParser;
    Interpreter : TXmlInterpreter;
    Visitor     : TXmlPrettyPrinter;
begin
  Parser      := TXmlParser.Create;
  Interpreter := TXmlInterpreter.Create;
  Visitor     := TXmlPrettyPrinter.Create;

  try
    Parser.Parse(FDocument.Text,Interpreter.XmlDoc);
    Interpreter.XmlDoc.Accept(Visitor);
    FDocument.Text := Visitor.Text;
    UpdateMemo;
  finally
    Parser.Free;
    Interpreter.Free;
    Visitor.Free;
  end;
end;


end.
