unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DabBoozePlant, DabDepartment, DabEmployee;

type
  TMainDlg = class(TForm)
    ExternalBtn: TButton;
    Memo: TMemo;
    InternalBtn: TButton;
    EventBtn: TButton;
    VisitorBtn: TButton;
    EmployeeListReportBtn: TButton;
    EmployeeSalarayReportBtn: TButton;
    procedure ExternalBtnClick(Sender: TObject);
    procedure InternalBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure EventBtnClick(Sender: TObject);
    procedure VisitorBtnClick(Sender: TObject);
    procedure EmployeeListReportBtnClick(Sender: TObject);
    procedure EmployeeSalarayReportBtnClick(Sender: TObject);
  private
    FPlants : TDabBoozePlants;
  public
  published
    procedure EmployeeOnProcessItem(Sender : TObject;
                                    CurrItem : TDabEmployee;
                                    var Stop : Boolean);
  end;

var
  MainDlg: TMainDlg;


implementation

{$R *.dfm}

uses
  Iterator, Visitor, Template, Strategy;

{ TMainDlg }

procedure TMainDlg.FormCreate(Sender: TObject);
begin
  FPlants := TDabBoozePlants.Create(Self,TDabBoozePlant);

  // Add some plants
  FPlants.AddNew('Ardbeg Distillery','Port Ellen, Islay, Scotland');
  FPlants.AddNew('Macallan Distillery','Easter Elchies, Scotland');
  FPlants.AddNew('Hoegaarden Brewery','Hoegaarden, Belgium');
  FPlants.AddNew('Coopers Brewery','Adelaide, South Australia');

  // Add some departments and employees to the plants
  FPlants[0].Employees.AddNew('Jim','Cooper','Chief taster and layabout',
                              TSalaryStrategy.Create(50000));
  FPlants[0].Departments.AddNew('Distilling');
  FPlants[0].Departments[0].Employees.AddNew('Fred','McBloggs','Stillman',
                                             TWageStrategy.Create(500));
  FPlants[0].Departments[0].Employees.AddNew('Jock','McBloggs',
                                             'Bloke who looks after the mash tun',
                                             TWageStrategy.Create(300));
  FPlants[0].Departments.AddNew('Sales');
  FPlants[0].Departments[1].Employees.AddNew('William','McGonagall','Manager and Poet',
                                             TSalaryStrategy.Create(35000));

  FPlants[1].Employees.AddNew('Jim','Cooper','Chief tonsil varnisher',
                              TSalaryStrategy.Create(45000));
  FPlants[1].Departments.AddNew('Distilling');
  FPlants[1].Departments[0].Employees.AddNew('Och Jock','McPlop','Stillman',
                                             TWageStrategy.Create(500));
  FPlants[1].Departments[0].Employees.AddNew('Jock','McBloggs (no relation)',
                                             'Assistant stillman',
                                             TWageStrategy.Create(350));
  FPlants[1].Departments.AddNew('Warehouse');
  FPlants[1].Departments[1].Employees.AddNew('Wee Wully','McTaggart','Cask lifter',
                                             TWageStrategy.Create(220));

  FPlants[2].Employees.AddNew('Jim','Cooper','Chief buyer',
                              TSalaryStrategy.Create(55000));
  FPlants[2].Departments.AddNew('Brewing');
  FPlants[2].Departments[0].Employees.AddNew('Jean','Gueuze','Brewing bloke',
                                             TWageStrategy.Create(250));
  FPlants[2].Departments[0].Employees.AddNew('Henri','Gueuze','Brewing bloke',
                                             TWageStrategy.Create(250));
  FPlants[2].Departments[0].Employees.AddNew('Wee Wully','McTaggart','Brewing bloke',
                                             TWageStrategy.Create(250));

  FPlants[3].Employees.AddNew('Jim','Cooper','No relation',
                              TSalaryStrategy.Create(12));
end;


procedure TMainDlg.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FPlants);
end;


procedure TMainDlg.ExternalBtnClick(Sender: TObject);
  var
    TempIterator : TEmployeeIterator;
begin
  // Most flexible, as can do whatever you like
  TempIterator := TEmployeeIterator.Create(FPlants[2].Departments[0].Employees);

  try
    Memo.Clear;
    TempIterator.First;

    while not TempIterator.IsDone do begin
      Memo.Lines.Add(TempIterator.CurrentItem.FullName + ',  ' +
                     TempIterator.CurrentItem.Position);
      TempIterator.Next;
    end;
  finally
    TempIterator.Free;
  end;
end;


procedure TMainDlg.InternalBtnClick(Sender: TObject);
  var
    TempIterator : TEmployeeIterator;
begin
  TempIterator := TEmployeeIterator.Create(FPlants[2].Departments[0].Employees);

  try
    TempIterator.Traverse(Memo.Lines);
  finally
    TempIterator.Free;
  end;
end;


procedure TMainDlg.EmployeeOnProcessItem(Sender : TObject;
                                         CurrItem : TDabEmployee;
                                         var Stop : Boolean);
begin
  Memo.Lines.Add(CurrItem.FullName + ',  ' +
                 CurrItem.Position);
end;


procedure TMainDlg.EventBtnClick(Sender: TObject);
  var
    TempIterator : TEmployeeIterator;
begin
  Memo.Clear;
  TempIterator := TEmployeeIterator.Create(FPlants[2].Departments[0].Employees);

  try
    TempIterator.OnProcessItem := EmployeeOnProcessItem;
    TempIterator.EventTraverse;
  finally
    TempIterator.Free;
  end;
end;


procedure TMainDlg.VisitorBtnClick(Sender: TObject);
  var
    TempVisitor : TListEmployees;
begin
  TempVisitor := TListEmployees.Create(Memo.Lines);

  try
    FPlants.Accept(TempVisitor);
  finally
    TempVisitor.Free;
  end;
end;


procedure TMainDlg.EmployeeListReportBtnClick(Sender: TObject);
begin
  TEmployeeListReport.Execute(Memo.Lines,FPlants);
end;


procedure TMainDlg.EmployeeSalarayReportBtnClick(Sender: TObject);
begin
  TEmployeePayReport.Execute(Memo.Lines,FPlants);
end;


end.
